<?php

/**
 * @file
 * CaseTrackerCase editing UI.
 *
 * We make very little use of the EntityAPI interface for this - preferring instead to use
 * views. That offers more flexibility to change a UI that will, more often than not,
 * be end-user facing.
 */

/**
 * UI controller.
 */
class ScrumSprintUIController extends EntityDefaultUIController {

  /**
   * Overrides hook_menu() defaults. Main reason for doing this is that
   * parent class hook_menu() is optimized for entity type administration.
   */
  public function hook_menu() {

    $items = array();
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%' . $this->entityType;

    // Change the add page menu to multiple types of entities
    $items[$this->path . '/add'] = array(
      'title' => t('Add a sprint'),
      'description' => t('Add a new sprint'),
      'page callback' => 'scrum_sprint_form_wrapper',
      'page arguments' => array(scrum_sprint_create()),
      'access callback' => 'scrum_sprint_access',
      'access arguments' => array('edit'),
      // 'type' => MENU_NORMAL_ITEM,
      // 'weight' => 20,
      'file' => 'admin/scrum_sprint.admin.inc',
      'file path' => drupal_get_path('module', $this->entityInfo['module'])
    );

    $sprint_path = 'scrum/sprint';
    $items[$sprint_path . '/' . $wildcard . '/edit'] = array(
      'page callback' => 'scrum_sprint_form_wrapper',
      'page arguments' => array(2),
      'access callback' => 'scrum_sprint_access',
      'access arguments' => array('edit', 2),
      'title' => 'Edit',
      'type' => MENU_LOCAL_TASK,
      'weight' => 0,
      'context' => MENU_CONTEXT_PAGE | MENU_CONTEXT_INLINE,
    );

    $items[$sprint_path . '/' . $wildcard . '/delete'] = array(
      'title' => 'Delete',
      'page callback' => 'scrum_sprint_delete_form_wrapper',
      'page arguments' => array(2),
      'access callback' => 'scrum_sprint_access',
      'access arguments' => array('edit', 2),
      'type' => MENU_LOCAL_TASK,
      'context' => MENU_CONTEXT_INLINE,
      'weight' => 10,
      'file' => 'admin/scrum_sprint.admin.inc',
      'file path' => drupal_get_path('module', $this->entityInfo['module'])
    );

    // Menu item for viewing scrum_sprints
    $items[$sprint_path . '/' . $wildcard] = array(
      'page callback' => 'scrum_sprint_page_view',
      'page arguments' => array(2),
      'access callback' => 'scrum_sprint_access',
      'access arguments' => array('view', 2),
    );

    $items[$sprint_path . '/' . $wildcard . '/view'] = array(
      'title' => 'View',
      'type' => MENU_DEFAULT_LOCAL_TASK,
      'weight' => -10,
    );

    return $items;
  }

  /**
   * Create the markup for the add CaseTrackerCase Entities page within the class
   * so it can easily be extended/overriden.
   */
  public function addPage() {
    $item = menu_get_item();
    $content = system_admin_menu_block($item);

    $item = array_shift($content);
    // Check to see if a project is being referenced, to pass the reference 
    $project_ref = arg(5);
    if(is_numeric($project_ref)) {
      drupal_goto($item['href'] . '/' . $project_ref);
    }
    else {
      drupal_goto($item['href']);
    }

    return theme('scrum_sprint_add_list', array('content' => $content));
  }

}

/**
 * Form callback wrapper: create or edit a scrum_sprint.
 *
 * @param $scrum_sprint
 *   The scrum_sprint object being edited by this form.
 *
 * @see scrum_sprint_edit_form()
 */
function scrum_sprint_form_wrapper($sprint) {
  return drupal_get_form('scrum_sprint_edit_form', $sprint);
}

/**
 * Form callback wrapper: delete a scrum_sprint.
 *
 * @param $sprint
 *   The scrum_sprint object being edited by this form.
 *
 * @see scrum_sprint_edit_form()
 */
function scrum_sprint_delete_form_wrapper($sprint) {
  // Add the breadcrumb for the form's location.
  //scrum_sprint_set_breadcrumb();
  return drupal_get_form('scrum_sprint_delete_form', $sprint);
}

/**
 * Form callback: create or edit a scrum_sprint.
 *
 * @param $sprint
 *   The scrum_sprint object to edit or for a create form an empty 
 *   scrum_sprint object with only a scrum_sprint type defined.
 */
function scrum_sprint_edit_form($form, &$form_state, $sprint) {

  $amount_of_projects = _casetracker_project_get_amount();
  if ($amount_of_projects == 0) {
    drupal_set_message(t('You need to have at least 1 project to relate your sprints to. If you wish, you can go to the <a href="@link">projects creation page</a> now to create one.'
            , array('@link' => base_path() . 'admin/content/casetracker/projects/add')), 'error');
  }

  casetracker_project_set_breadcrumb();

  if (!empty($sprint->sid)) {
    drupal_set_title(t('@sprint (editing sprint info)', array('@sprint' => $sprint->title)));
  }

  // Add the default field elements.
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => isset($sprint->title) ? $sprint->title : '',
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => 0,
  );
  $form['project'] = array(
    '#type' => 'textfield',
    '#title' => t('Project'),
    '#default_value' => isset($sprint->project_id) ? $sprint->project_id : '',
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => -1,
  );
  $form['start_date'] = array(
    '#type' => 'date_popup',
    '#date_format' =>  variable_get('date_format_short', 'm/d/Y'),
    '#title' => t('Start date'),
    '#default_value' => isset($sprint->start_date) ? $sprint->start_date : '',
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => 1,
  );
  $form['finish_date'] = array(
    '#type' => 'date_popup',
    '#title' => t('Finish date'),
    '#default_value' => isset($sprint->finish_date) ? $sprint->finish_date : '',
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => 2,
  );

  // Add the field related form elements.
  $form_state['scrum_sprint'] = $sprint;
  field_attach_form('scrum_sprint', $sprint, $form, $form_state);

  // If the entity is being created, try to prepopulate the project reference
  // from the URL argument
  // if (isset($sprint->is_new) && $sprint->is_new) {
  //   // If the project was informed via URL argument, prepopulate the field
  //   $pid = arg(6);
  //   if (!empty($pid)) {
  //     $project = casetracker_project_load($pid);
  //     if ($project) {
  //       $case_wrapper = entity_metadata_wrapper('scrum_sprint', $sprint);
  //       $selected_project = $project->title . ' (' . $pid . ')';
  //       $form['field_casetracker_project_ref']['und']['#default_value'] = $selected_project;
  //       drupal_set_title(t('Add Case to project "@title"', array('@title' => $project->title)));
  //     }
  //   }
  // }

  $form['uid'] = array(
    '#type' => 'value',
    '#value' => isset($sprint->uid) ? $sprint->uid : '',
  );

  $form['actions'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('form-actions')),
    '#weight' => 400,
  );

  // We add the form's #submit array to this button along with the actual submit
  // handler to preserve any submit handlers added by a form callback_wrapper.
  $submit = array();

  if (!empty($form['#submit'])) {
    $submit += $form['#submit'];
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save sprint'),
    '#submit' => $submit + array('scrum_sprint_edit_form_submit'),
  );

  // We append the validate handler to #validate in sprint a form callback_wrapper
  // is used to add validate handlers earlier.
  $form['#validate'][] = 'scrum_sprint_edit_form_validate';
  return $form;
}

/**
 * Form API validate callback for the scrum_sprint form
 */
function scrum_sprint_edit_form_validate(&$form, &$form_state) {
  $sprint = $form_state['scrum_sprint'];

  // Notify field widgets to validate their data.
  field_attach_form_validate('scrum_sprint', $sprint, $form, $form_state);
}

/**
 * Form API submit callback for the scrum_sprint form.
 * 
 * @todo remove hard-coded link
 */
function scrum_sprint_edit_form_submit(&$form, &$form_state) {

  $sprint = entity_ui_controller('scrum_sprint')->entityFormSubmitBuildEntity($form, $form_state);

  if ($sprint->is_new = isset($sprint->is_new) ? $sprint->is_new : 0) {
    $sprint->created = time();
    global $user;
    $sprint->uid = $user->uid;
  }
  $sprint->changed = time();

  $sprint->save();

  // Save the scrum_sprint and go back to the list of scrum_sprints
  // Add in created and changed times.
  if ($sprint->is_new = isset($sprint->is_new) ? $sprint->is_new : 0) {
    drupal_set_message(t('The sprint "!title" was created successfully.', array('!title' => l($sprint->title, '`/sprint/' . $sprint->sid))));
  }
  else {
    drupal_set_message(t('The sprint "!title" was updated successfully.', array('!title' => l($sprint->title, 'scrum/sprint/' . $sprint->sid))));
  }
}

/**
 * Form API submit callback for the delete button.
 * 
 * @todo Remove hard-coded path
 */
function scrum_sprint_form_submit_delete(&$form, &$form_state) {
  $form_state['redirect'] = 'scrum/sprint/' . $form_state['scrum_sprint']->pid . '/delete';
}

/**
 * Form callback: confirmation form for deleting a scrum_sprint.
 *
 * @param $sprint
 *   The scrum_sprint to delete
 *
 * @see confirm_form()
 */
function scrum_sprint_delete_form($form, &$form_state, $sprint) {

  scrum_sprint_set_breadcrumb();

  $form_state['scrum_sprint'] = $sprint;

  $form['#submit'][] = 'scrum_sprint_delete_form_submit';

  $form = confirm_form($form, t('Are you sure you want to delete sprint "@title"?', array('@title' => $sprint->title)), 'admin/content/casetracker/scrum-sprints', '<p>' . t('This action cannot be undone.') . '</p>', t('Delete'), t('Cancel'), 'confirm'
  );

  return $form;
}

/**
 * Submit callback for scrum_sprint_delete_form
 */
function scrum_sprint_delete_form_submit($form, &$form_state) {
  $sprint = $form_state['scrum_sprint'];

  scrum_sprint_delete($sprint);

  drupal_set_message(t('The sprint "%title" has been deleted.', array('%title' => $sprint->title)));
  watchdog('scrum_sprint', 'Deleted sprint "%title".', array('%title' => $sprint->title));
}

/**
 * Page to add CaseTrackerCase Entities.
 *
 * @todo Pass this through a proper theme function
 */
function scrum_sprint_add_page() {
  $controller = entity_ui_controller('scrum_sprint');
  return $controller->addPage();
}

/**
 * Displays the list of available scrum_sprint types for scrum_sprint creation.
 *
 * @ingroup themeable
 */
function theme_scrum_sprint_add_list($variables) {
  $content = $variables['content'];
  $output = '';
  $output = '<dl class="scrum-sprint-type-list">';
  foreach ($content as $item) {
    $output .= '<dt>' . l($item['title'], $item['href']) . '</dt>';
    $output .= '<dd>' . filter_xss_admin($item['description']) . '</dd>';
  }
  $output .= '</dl>';
  return $output;
}